@@ -17,15 +17,21 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- *  $Id$
  */
 
+#if !defined (__EPHY_EPIPHANY_H_INSIDE__) && !defined (EPIPHANY_COMPILATION)
+#error "Only <epiphany/epiphany.h> can be included directly."
+#endif
+
 #ifndef EPHY_FILE_HELPERS_H
 #define EPHY_FILE_HELPERS_H
 
 #include <glib.h>
 #include <gio/gio.h>
-#include <gtk/gtkwidget.h>
+#include <gtk/gtk.h>
+
+/* From libgnome */
+#define GNOME_DOT_GNOME ".gnome2"
 
 extern GQuark ephy_file_helpers_error_quark;
 #define EPHY_FILE_HELPERS_ERROR_QUARK	(ephy_file_helpers_error_quark)
