@@ -3,6 +3,7 @@
  *  Copyright © 2002  Ricardo Fernández Pascual
  *  Copyright © 2003, 2004  Marco Pesenti Gritti
  *  Copyright © 2003, 2004, 2005  Christian Persch
+ *  Copyright © 2008  Xan López
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -18,12 +19,10 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- *  $Id$
  */
 
 #include "config.h"
 
-#include "ephy-icon-entry.h"
 #include "ephy-tree-model-node.h"
 #include "ephy-location-entry.h"
 #include "ephy-marshal.h"
@@ -44,17 +43,13 @@
 
 struct _EphyLocationEntryPrivate
 {
-	EphyIconEntry *icon_entry;
-	GtkWidget *icon_ebox;
-	GtkWidget *icon;
-	GtkWidget *lock_ebox;
-	GtkWidget *lock;
+	GtkWidget *entry;
+	char *lock_stock_id;
 	GdkPixbuf *favicon;
 	GdkColor secure_bg_colour;
 	GdkColor secure_fg_colour;
 
-	GtkCellRenderer *extracell;
-	GRegex *regex;
+	GSList *search_terms;
 
 	char *before_completion;
 	char *saved_text;
@@ -76,6 +71,7 @@ struct _EphyLocationEntryPrivate
 	guint secure : 1;
 	guint apply_colours : 1;
 	guint needs_reset : 1;
+	guint show_lock : 1;
 };
 
 static const GtkTargetEntry url_drag_types [] =
@@ -92,13 +88,16 @@ static void ephy_location_entry_class_init (EphyLocationEntryClass *klass);
 static void ephy_location_entry_init (EphyLocationEntry *le);
 static gboolean ephy_location_entry_reset_internal (EphyLocationEntry *, gboolean);
 
-static void extracell_data_func (GtkCellLayout *cell_layout,
+static void textcell_data_func (GtkCellLayout *cell_layout,
 				GtkCellRenderer *cell,
 				GtkTreeModel *tree_model,
 				GtkTreeIter *iter,
 				gpointer data);
-
-static GObjectClass *parent_class = NULL;
+static void extracell_data_func (GtkCellLayout *cell_layout,
+				 GtkCellRenderer *cell,
+				 GtkTreeModel *tree_model,
+				 GtkTreeIter *iter,
+				 gpointer data);
 
 enum signalsEnum
 {
@@ -114,33 +113,7 @@ static gint signals[LAST_SIGNAL] = { 0 };
 #define EPHY_LOC_HISTORY_XML_ROOT "ephy_location_history"
 #define EPHY_LOC_HISTORY_XML_VERSION "0.1"
 
-GType
-ephy_location_entry_get_type (void)
-{
-	static GType type = 0;
-
-	if (G_UNLIKELY (type == 0))
-	{
-		const GTypeInfo our_info =
-		{
-			sizeof (EphyLocationEntryClass),
-			NULL,
-			NULL,
-			(GClassInitFunc) ephy_location_entry_class_init,
-			NULL,
-			NULL,
-			sizeof (EphyLocationEntry),
-			0,
-			(GInstanceInitFunc) ephy_location_entry_init
-		};
-
-		type = g_type_register_static (GTK_TYPE_TOOL_ITEM,
-					       "EphyLocationEntry",
-					       &our_info, 0);
-	}
-
-	return type;
-}
+G_DEFINE_TYPE (EphyLocationEntry, ephy_location_entry, GTK_TYPE_TOOL_ITEM)
 
 static void
 ephy_location_entry_style_set (GtkWidget *widget,
@@ -154,16 +127,12 @@ ephy_location_entry_style_set (GtkWidget *widget,
 	char *theme;
 	gboolean is_a11y_theme;
 
-	if (GTK_WIDGET_CLASS (parent_class)->style_set)
+	if (GTK_WIDGET_CLASS (ephy_location_entry_parent_class)->style_set)
 	{
-		GTK_WIDGET_CLASS (parent_class)->style_set (widget, previous_style);
+		GTK_WIDGET_CLASS (ephy_location_entry_parent_class)->style_set (widget, previous_style);
 	}
 
 	title_fg_colour = widget->style->text[GTK_STATE_INSENSITIVE];
-	g_object_set (entry->priv->extracell, 
-			"foreground-gdk", &title_fg_colour,
-			"foreground-set", TRUE,
-			NULL);
 
 	settings = gtk_settings_get_for_screen (gtk_widget_get_screen (widget));
 	g_object_get (settings, "gtk-theme-name", &theme, NULL);
@@ -203,6 +172,17 @@ ephy_location_entry_style_set (GtkWidget *widget,
 	ephy_location_entry_set_secure (entry, priv->secure);
 }
 
+inline static void
+free_search_terms (GSList *search_terms)
+{
+	GSList *iter;
+	
+	for (iter = search_terms; iter != NULL; iter = iter->next)
+		g_regex_unref ((GRegex*)iter->data);
+	
+	g_slist_free (search_terms);
+}
+
 static void
 ephy_location_entry_finalize (GObject *object)
 {
@@ -216,13 +196,13 @@ ephy_location_entry_finalize (GObject *object)
 		g_object_unref (priv->favicon);
 	}
 	
-	if (priv->regex)
+	if (priv->search_terms)
 	{
-		g_regex_unref (priv->regex);
-		priv->regex = NULL;
+		free_search_terms (priv->search_terms);
+		priv->search_terms = NULL;
 	}
 
-	parent_class->finalize (object);
+	G_OBJECT_CLASS (ephy_location_entry_parent_class)->finalize (object);
 }
 
 static void
@@ -231,8 +211,6 @@ ephy_location_entry_class_init (EphyLocationEntryClass *klass)
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 	GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
 
-	parent_class = g_type_class_peek_parent (klass);
-
 	object_class->finalize = ephy_location_entry_finalize;
 
 	widget_class->style_set = ephy_location_entry_style_set;
@@ -300,23 +278,25 @@ update_address_state (EphyLocationEntry *entry)
 	EphyLocationEntryPrivate *priv = entry->priv;
 	const char *text;
 
-	text = gtk_entry_get_text (GTK_ENTRY (priv->icon_entry->entry));
+	text = gtk_entry_get_text (GTK_ENTRY (priv->entry));
 	priv->original_address = text != NULL &&
 				 g_str_hash (text) == priv->hash;
 }
 
 static void
-update_favicon (EphyLocationEntry *entry)
+update_favicon (EphyLocationEntry *lentry)
 {
-	EphyLocationEntryPrivate *priv = entry->priv;
-	GtkImage *image = GTK_IMAGE (priv->icon);
+	EphyLocationEntryPrivate *priv = lentry->priv;
+	GtkEntry *entry = GTK_ENTRY (priv->entry);
 
 	/* Only show the favicon if the entry's text is the
 	 * address of the current page.
 	 */
 	if (priv->favicon != NULL && priv->original_address)
 	{
-		gtk_image_set_from_pixbuf (image, priv->favicon);
+		gtk_entry_set_icon_from_pixbuf (entry,
+						GTK_ENTRY_ICON_PRIMARY,
+						priv->favicon);
 	}
 	else
 	{
@@ -324,9 +304,9 @@ update_favicon (EphyLocationEntry *entry)
 		 * the page MIME type, though text/html should be good enough
 		 * most of the time. See #337140
 		 */
-		gtk_image_set_from_icon_name (image,
-					      "text-html",
-					      GTK_ICON_SIZE_MENU);
+		gtk_entry_set_icon_from_icon_name (entry,
+						   GTK_ENTRY_ICON_PRIMARY,
+						   "text-html");
 	}
 }
 
@@ -348,24 +328,103 @@ editable_changed_cb (GtkEditable *editable,
 		priv->can_redo = FALSE;
 	}	
 	
-	if (priv->regex)
+	if (priv->search_terms)
 	{
-		g_regex_unref (priv->regex);
-		priv->regex = NULL;
+		free_search_terms (priv->search_terms);
+		priv->search_terms = NULL;
 	}
-	
+
 	text = gtk_entry_get_text (GTK_ENTRY (editable));
 
+	/*
+	 * user is specifying a regular expression, so we will
+	 * have only one search term
+	 */
 	if (g_str_has_prefix (text, "re:"))
+	{
+		GRegex *regex;
 		pattern = g_strdup (text+3);
+		regex = g_regex_new (pattern,
+				     G_REGEX_CASELESS | G_REGEX_OPTIMIZE,
+				     G_REGEX_MATCH_NOTEMPTY, NULL);
+		priv->search_terms = g_slist_append (priv->search_terms, regex);
+		g_free (pattern);
+	}
 	else
-		pattern = g_regex_escape_string (text, -1);
+	{
+		const char *current;
+		const char *ptr;
+		char *tmp;
+		char *term;
+		GRegex *term_regex;
+		GRegex *quote_regex;
+		gint count;
+		gboolean inside_quotes = FALSE;
 
-	priv->regex = g_regex_new (pattern, 
-				G_REGEX_CASELESS | G_REGEX_OPTIMIZE, 
-				G_REGEX_MATCH_NOTEMPTY, NULL);
+		quote_regex = g_regex_new ("\"", G_REGEX_OPTIMIZE,
+					   G_REGEX_MATCH_NOTEMPTY, NULL);
+		
+		/*
+		 * This code loops through the string using pointer arythmetics.
+		 * Although the string we are handling may contain UTF-8 chars
+		 * this works because only ASCII chars affect what is actually
+		 * copied from the string as a search term.
+		 */
+		for (count = 0, current = ptr = text; ptr[0] != '\0'; ptr++, count++)
+		{
+			/*
+			 * If we found a double quote character; we will 
+			 * consume bytes up until the next quote, or
+			 * end of line;
+			 */
+			if (ptr[0] == '"')
+				inside_quotes = !inside_quotes;
 
-	g_free (pattern);
+			/*
+			 * If we found a space, and we are not looking for a
+			 * closing double quote, or if the next char is the
+			 * end of the string, append what we have already as
+			 * a search term.
+			 */
+			if (((ptr[0] == ' ') && (!inside_quotes)) || ptr[1] == '\0')
+			{
+				/*
+				 * We special-case the end of the line because
+				 * we would otherwise not copy the last character
+				 * of the search string, since the for loop will
+				 * stop before that.
+				 */
+				if (ptr[1] == '\0')
+					count++;
+				
+				/*
+				 * remove quotes, and quote any regex-sensitive
+				 * characters
+				 */
+				tmp = g_regex_escape_string (current, count);
+				term = g_regex_replace (quote_regex, tmp, -1, 0,
+							"", G_REGEX_MATCH_NOTEMPTY, NULL);
+				g_strstrip (term);
+				g_free (tmp);
+
+				/* we don't want empty search terms */
+				if (term[0] != '\0')
+				{
+					term_regex = g_regex_new (term,
+								  G_REGEX_CASELESS | G_REGEX_OPTIMIZE,
+								  G_REGEX_MATCH_NOTEMPTY, NULL);
+					priv->search_terms = g_slist_append (priv->search_terms, term_regex);
+				}
+				g_free (term);
+
+				 /* count will be incremented by the for loop */
+				count = -1;
+				current = ptr + 1;
+			}
+		}
+
+		g_regex_unref (quote_regex);
+	}
 
 	update_favicon (entry);
 
@@ -464,8 +523,8 @@ match_selected_cb (GtkEntryCompletion *completion,
 			     state == (GDK_CONTROL_MASK | GDK_SHIFT_MASK));
 
 	ephy_location_entry_set_location (entry, item, NULL);
-	gtk_im_context_reset (GTK_ENTRY (entry->priv->icon_entry->entry)->im_context);
-	g_signal_emit_by_name (priv->icon_entry->entry, "activate");
+	gtk_im_context_reset (GTK_ENTRY (entry->priv->entry)->im_context);
+	g_signal_emit_by_name (priv->entry, "activate");
 
 	g_free (item);
 
@@ -541,7 +600,7 @@ entry_clear_activate_cb (GtkMenuItem *item,
 	EphyLocationEntryPrivate *priv = entry->priv;
 
 	priv->block_update = TRUE;
-	gtk_entry_set_text (GTK_ENTRY (priv->icon_entry->entry), "");
+	gtk_entry_set_text (GTK_ENTRY (priv->entry), "");
 	priv->block_update = FALSE;
 	priv->user_changed = TRUE;
 }
@@ -584,7 +643,7 @@ entry_populate_popup_cb (GtkEntry *entry,
 	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (clear_menuitem), image);
 	g_signal_connect (clear_menuitem , "activate",
 			  G_CALLBACK (entry_clear_activate_cb), lentry);
-	is_editable = gtk_editable_get_editable (GTK_EDITABLE (priv->icon_entry->entry));
+	is_editable = gtk_editable_get_editable (GTK_EDITABLE (priv->entry));
 	gtk_widget_set_sensitive (clear_menuitem, is_editable);
 	gtk_widget_show (clear_menuitem);
 
@@ -764,11 +823,19 @@ favicon_create_drag_pixmap (EphyLocationEntry *entry,
 static void
 favicon_drag_begin_cb (GtkWidget *widget,
 		       GdkDragContext *context,
-		       EphyLocationEntry *entry)
+		       EphyLocationEntry *lentry)
 {
 	GdkPixmap *pixmap;
+	GtkEntry *entry;
+	gint index;
 
-	pixmap = favicon_create_drag_pixmap (entry, widget);
+	entry = GTK_ENTRY (widget);
+	
+	index = gtk_entry_get_current_icon_drag_source (entry);
+	if (index != GTK_ENTRY_ICON_PRIMARY)
+		return;
+
+	pixmap = favicon_create_drag_pixmap (lentry, widget);
 
 	if (pixmap != NULL)
 	{
@@ -785,50 +852,50 @@ favicon_drag_data_get_cb (GtkWidget *widget,
 			  GtkSelectionData *selection_data,
 			  guint info,
 			  guint32 time,
-			  EphyLocationEntry *entry)
+			  EphyLocationEntry *lentry)
 {
+	gint index;
+	GtkEntry *entry;
+
 	g_assert (widget != NULL);
 	g_return_if_fail (context != NULL);
 
-	ephy_dnd_drag_data_get (widget, context, selection_data,
-		time, entry, each_url_get_data_binder);
+	entry = GTK_ENTRY (widget);
+
+	index = gtk_entry_get_current_icon_drag_source (entry);
+	if (index == GTK_ENTRY_ICON_PRIMARY)
+	{
+		ephy_dnd_drag_data_get (widget, context, selection_data,
+					time, lentry, each_url_get_data_binder);
+	}
 }
 
 static gboolean
-icon_button_press_event_cb (GtkWidget *ebox,
+icon_button_press_event_cb (GtkWidget *entry,
+			    GtkEntryIconPosition position,
 			    GdkEventButton *event,
-			    EphyLocationEntry *entry)
+			    EphyLocationEntry *lentry)
 {
 	guint state = event->state & gtk_accelerator_get_default_mod_mask ();
 
 	if (event->type == GDK_BUTTON_PRESS && 
-	    event->button == 1 && 
+	    event->button == 1 &&
 	    state == 0 /* left */)
 	{
-		GtkWidget *toplevel;
-		EphyLocationEntryPrivate *priv = entry->priv;
+		if (position == GTK_ENTRY_ICON_PRIMARY)
+		{
+			GtkWidget *toplevel;
 		
-		toplevel = gtk_widget_get_toplevel (GTK_WIDGET (entry));
-		gtk_window_set_focus (GTK_WINDOW(toplevel),
-			      	      priv->icon_entry->entry);
+			toplevel = gtk_widget_get_toplevel (GTK_WIDGET (entry));
+			gtk_window_set_focus (GTK_WINDOW (toplevel), entry);
 
-		gtk_editable_select_region (GTK_EDITABLE (priv->icon_entry->entry), 
-					    0, -1);
-
-		return TRUE;
-	}
-
-	return FALSE;
-}
-
-static gboolean
-lock_button_press_event_cb (GtkWidget *ebox,
-			    GdkEventButton *event,
-			    EphyLocationEntry *entry)
-{
-	if (event->type == GDK_BUTTON_PRESS && event->button == 1 /* left */)
-	{
-		g_signal_emit (entry, signals[LOCK_CLICKED], 0);
+			gtk_editable_select_region (GTK_EDITABLE (entry), 
+						    0, -1);
+		}
+		else
+		{
+			g_signal_emit (lentry, signals[LOCK_CLICKED], 0);
+		}
 
 		return TRUE;
 	}
@@ -837,76 +904,57 @@ lock_button_press_event_cb (GtkWidget *ebox,
 }
 
 static void
-ephy_location_entry_construct_contents (EphyLocationEntry *entry)
+ephy_location_entry_construct_contents (EphyLocationEntry *lentry)
 {
-	EphyLocationEntryPrivate *priv = entry->priv;
-	GtkWidget *alignment;
+	EphyLocationEntryPrivate *priv = lentry->priv;
+	GtkTargetList *targetlist;
 
-	LOG ("EphyLocationEntry constructing contents %p", entry);
+	LOG ("EphyLocationEntry constructing contents %p", lentry);
 
-	alignment = gtk_alignment_new (0.0, 0.5, 1.0, 0.0);
-	gtk_alignment_set_padding (GTK_ALIGNMENT (alignment), 0, 0, 1, 1);
-	gtk_container_add (GTK_CONTAINER (entry), alignment);
+	priv->entry = gtk_entry_new ();
+	gtk_container_add (GTK_CONTAINER (lentry), priv->entry);
 
-	priv->icon_entry = EPHY_ICON_ENTRY (ephy_icon_entry_new ());
-	gtk_container_add (GTK_CONTAINER (alignment),
-			   GTK_WIDGET (priv->icon_entry));
+	/* Favicon */
+	targetlist = gtk_target_list_new (url_drag_types,
+					  G_N_ELEMENTS (url_drag_types));
+	gtk_entry_set_icon_drag_source (GTK_ENTRY (priv->entry),
+					GTK_ENTRY_ICON_PRIMARY,
+					targetlist,
+					GDK_ACTION_ASK | GDK_ACTION_COPY | GDK_ACTION_LINK);
+	gtk_target_list_unref (targetlist);
 
-	priv->icon_ebox = gtk_event_box_new ();
-	gtk_container_set_border_width (GTK_CONTAINER (priv->icon_ebox), 2);
-	gtk_event_box_set_visible_window (GTK_EVENT_BOX (priv->icon_ebox), FALSE);
-	gtk_widget_add_events (priv->icon_ebox, GDK_BUTTON_PRESS_MASK |
-			      			GDK_BUTTON_RELEASE_MASK |
-						GDK_POINTER_MOTION_HINT_MASK);
-	gtk_drag_source_set (priv->icon_ebox, GDK_BUTTON1_MASK,
-			     url_drag_types, G_N_ELEMENTS (url_drag_types),
-			     GDK_ACTION_ASK | GDK_ACTION_COPY | GDK_ACTION_LINK);
-	g_signal_connect (priv->icon_ebox, "button-press-event",
-			  G_CALLBACK (icon_button_press_event_cb), entry);
+	gtk_entry_set_icon_tooltip_text (GTK_ENTRY (priv->entry),
+					 GTK_ENTRY_ICON_PRIMARY,
+					 _("Drag and drop this icon to create a link to this page"));
 
-	gtk_widget_set_tooltip_text (priv->icon_ebox,
-				     _("Drag and drop this icon to create a link to this page"));
+	gtk_entry_set_icon_activatable (GTK_ENTRY (priv->entry),
+					GTK_ENTRY_ICON_PRIMARY,
+					TRUE);
+	gtk_entry_set_icon_from_stock (GTK_ENTRY (priv->entry),
+				       GTK_ENTRY_ICON_SECONDARY,
+				       STOCK_LOCK_INSECURE);
+	gtk_entry_set_icon_activatable (GTK_ENTRY (priv->entry),
+					GTK_ENTRY_ICON_SECONDARY,
+					TRUE);
 
-	priv->icon = gtk_image_new ();
-	gtk_container_add (GTK_CONTAINER (priv->icon_ebox), priv->icon);
+	g_object_connect (priv->entry,
+			  "signal::icon-press", G_CALLBACK (icon_button_press_event_cb), lentry,
+			  "signal::populate-popup", G_CALLBACK (entry_populate_popup_cb), lentry,
+			  "signal::key-press-event", G_CALLBACK (entry_key_press_cb), lentry,
+			  "signal::changed", G_CALLBACK (editable_changed_cb), lentry,
+			  "signal::drag-motion", G_CALLBACK (entry_drag_motion_cb), lentry,
+			  "signal::drag-drop", G_CALLBACK (entry_drag_drop_cb), lentry,
+			  "signal::drag-data-get", G_CALLBACK (favicon_drag_data_get_cb), lentry,
+			  NULL);
+			  
+	g_signal_connect_after (priv->entry, "key-press-event",
+				G_CALLBACK (entry_key_press_after_cb), lentry);
+	g_signal_connect_after (priv->entry, "activate",
+				G_CALLBACK (entry_activate_after_cb), lentry);
+	g_signal_connect_after (priv->entry, "drag-begin",
+				G_CALLBACK (favicon_drag_begin_cb), lentry);
 
-	ephy_icon_entry_pack_widget (priv->icon_entry, priv->icon_ebox, TRUE);
-
-	priv->lock_ebox = gtk_event_box_new ();
-	gtk_container_set_border_width (GTK_CONTAINER (priv->lock_ebox), 2);
-	gtk_widget_add_events (priv->lock_ebox, GDK_BUTTON_PRESS_MASK);
-	gtk_event_box_set_visible_window (GTK_EVENT_BOX (priv->lock_ebox), FALSE);
-
-	priv->lock = gtk_image_new_from_stock (STOCK_LOCK_INSECURE,
-					       GTK_ICON_SIZE_MENU);
-	gtk_container_add (GTK_CONTAINER (priv->lock_ebox), priv->lock);
-
-	ephy_icon_entry_pack_widget (priv->icon_entry, priv->lock_ebox, FALSE);
-
-	g_signal_connect (priv->icon_ebox, "drag-data-get",
-			  G_CALLBACK (favicon_drag_data_get_cb), entry);
-	g_signal_connect (priv->icon_ebox, "drag-begin",
-			  G_CALLBACK (favicon_drag_begin_cb), entry);
-
-	g_signal_connect (priv->lock_ebox, "button-press-event",
-			  G_CALLBACK (lock_button_press_event_cb), entry);
-
-	g_signal_connect (priv->icon_entry->entry, "populate_popup",
-			  G_CALLBACK (entry_populate_popup_cb), entry);
-	g_signal_connect (priv->icon_entry->entry, "key-press-event",
-			  G_CALLBACK (entry_key_press_cb), entry);
-	g_signal_connect_after (priv->icon_entry->entry, "key-press-event",
-				G_CALLBACK (entry_key_press_after_cb), entry);
-	g_signal_connect_after (priv->icon_entry->entry, "activate",
-				G_CALLBACK (entry_activate_after_cb), entry);
-	g_signal_connect (priv->icon_entry->entry, "changed",
-			  G_CALLBACK (editable_changed_cb), entry);
-	g_signal_connect (priv->icon_entry->entry, "drag-motion",
-			  G_CALLBACK (entry_drag_motion_cb), entry);
-	g_signal_connect (priv->icon_entry->entry, "drag-drop",
-			  G_CALLBACK (entry_drag_drop_cb), entry);
-
-	gtk_widget_show_all (alignment);
+	gtk_widget_show (priv->entry);
 }
 
 static void
@@ -922,7 +970,8 @@ ephy_location_entry_init (EphyLocationEntry *le)
 	p->user_changed = FALSE;
 	p->block_update = FALSE;
 	p->saved_text = NULL;
-	p->regex = NULL;
+	p->search_terms = NULL;
+	p->show_lock = FALSE;
 	
 	ephy_location_entry_construct_contents (le);
 
@@ -949,14 +998,111 @@ cursor_on_match_cb  (GtkEntryCompletion *completion,
 			    &item, -1);
 	entry = gtk_entry_completion_get_entry (completion);
 
+	/* Prevent the update so we keep the highlight from our input.
+	 * See textcell_data_func().
+	 */
+	le->priv->block_update = TRUE;
 	gtk_entry_set_text (GTK_ENTRY (entry), item);
 	gtk_editable_set_position (GTK_EDITABLE (entry), -1);
+	le->priv->block_update = FALSE;
 
 	g_free (item);
 
 	return TRUE;
 }
 
+static void
+textcell_data_func (GtkCellLayout *cell_layout,
+			GtkCellRenderer *cell,
+			GtkTreeModel *tree_model,
+			GtkTreeIter *iter,
+			gpointer data)
+{
+	EphyLocationEntryPrivate *priv;
+	PangoAttrList *list;
+	PangoAttribute *att;
+	GMatchInfo *match;
+
+	int start, end;
+
+	char *ctext;
+	char *title;
+	char *url;
+
+	GtkStyle *style;
+	GdkColor color;
+
+	GValue text = { 0, };
+
+	priv = EPHY_LOCATION_ENTRY (data)->priv;
+	gtk_tree_model_get (tree_model, iter,
+			priv->text_col, &title,
+			priv->url_col, &url,
+			-1);
+
+	list = pango_attr_list_new ();
+
+	if (url)
+	{
+		ctext = g_strdup_printf ("%s\n%s", title, url);
+
+		style = gtk_widget_get_style (priv->entry);
+		color = style->text[GTK_STATE_INSENSITIVE];
+
+		att = pango_attr_foreground_new
+			(color.red, color.green, color.blue);
+		att->start_index = strlen (title)+1;
+
+		pango_attr_list_insert (list, att);
+	}
+	else
+	{
+		ctext = title;
+	}
+
+	if (priv->search_terms)
+	{
+		GSList *iter;
+		GRegex *regex;
+
+		for (iter = priv->search_terms; iter != NULL; iter = iter->next)
+		{
+			regex = (GRegex*) iter->data;
+			g_regex_match (regex, ctext, G_REGEX_MATCH_NOTEMPTY, &match);
+
+			while (g_match_info_matches (match))
+			{
+				g_match_info_fetch_pos (match, 0, &start, &end);
+
+				att = pango_attr_weight_new (PANGO_WEIGHT_BOLD);
+				att->start_index = start;
+				att->end_index = end;
+
+				pango_attr_list_insert (list, att);
+				g_match_info_next (match, NULL);
+			}
+
+			g_match_info_free (match);
+			match = NULL;
+		}
+
+	}
+
+	g_object_set (cell,
+		      "attributes", list,
+		      NULL);
+
+	g_value_init (&text, G_TYPE_STRING);
+	g_value_take_string (&text, ctext);
+	g_object_set_property (G_OBJECT (cell), "text", &text);
+	g_value_unset (&text);
+
+	pango_attr_list_unref (list);
+
+	g_free (title);
+	g_free (url);
+}
+
 static void
 extracell_data_func (GtkCellLayout *cell_layout,
 			GtkCellRenderer *cell,
@@ -964,43 +1110,70 @@ extracell_data_func (GtkCellLayout *cell_layout,
 			GtkTreeIter *iter,
 			gpointer data)
 {
-	char *cdata;
-	GValue visible = { 0, };
-	GValue text = { 0, };
 	EphyLocationEntryPrivate *priv;
+	gboolean is_bookmark = FALSE;
+	GValue visible = { 0, };
 
 	priv = EPHY_LOCATION_ENTRY (data)->priv;
+	gtk_tree_model_get (tree_model, iter,
+			    priv->extra_col, &is_bookmark,
+			    -1);
 
-	gtk_tree_model_get (tree_model, iter, priv->extra_col, &cdata, -1);
+	if (is_bookmark)
+		g_object_set (cell,
+			      "stock-id", EPHY_STOCK_BOOKMARK,
+			      NULL);
 
-	g_value_init (&text, G_TYPE_STRING);
 	g_value_init (&visible, G_TYPE_BOOLEAN);
-	
-	g_value_take_string (&text, cdata);
-	g_value_set_boolean (&visible, (cdata != NULL));
-
-	g_object_set_property (G_OBJECT (cell), "text", &text);
+	g_value_set_boolean (&visible, is_bookmark);
 	g_object_set_property (G_OBJECT (cell), "visible", &visible);
-	
-	g_value_unset (&text);
 	g_value_unset (&visible);
 }
 
+/**
+ * ephy_location_entry_set_match_func:
+ * @entry: an #EphyLocationEntry widget
+ * @match_func: a #GtkEntryCompletionMatchFunc
+ * @user_data: user_data to pass to the @match_func
+ * @notify: a #GDestroyNotify, like the one given to
+ * gtk_entry_completion_set_match_func
+ *
+ * Sets the match_func for the internal #GtkEntryCompletion to @match_func.
+ *
+ **/
 void
-ephy_location_entry_set_match_func (EphyLocationEntry *le, 
+ephy_location_entry_set_match_func (EphyLocationEntry *entry, 
 				GtkEntryCompletionMatchFunc match_func,
 				gpointer user_data,
 				GDestroyNotify notify)
 {
-	EphyLocationEntryPrivate *priv = le->priv;
+	EphyLocationEntryPrivate *priv = entry->priv;
 	GtkEntryCompletion *completion;
 	
-	completion = gtk_entry_get_completion (GTK_ENTRY (priv->icon_entry->entry));
+	completion = gtk_entry_get_completion (GTK_ENTRY (priv->entry));
 	gtk_entry_completion_set_match_func (completion, match_func, user_data, notify);
 }
 
+/**
+ * ephy_location_entry_set_completion:
+ * @entry: an #EphyLocationEntry widget
+ * @model: the #GtkModel for the completion
+ * @text_col: column id to access #GtkModel relevant data
+ * @action_col: column id to access #GtkModel relevant data
+ * @keywords_col: column id to access #GtkModel relevant data
+ * @relevance_col: column id to access #GtkModel relevant data
+ * @url_col: column id to access #GtkModel relevant data
+ * @extra_col: column id to access #GtkModel relevant data
+ * @favicon_col: column id to access #GtkModel relevant data
+ *
+ * Initializes @entry to have a #GtkEntryCompletion using @model as the
+ * internal #GtkModel. The *_col arguments are for internal data retrieval from
+ * @model, like when setting the text property of one of the #GtkCellRenderer
+ * of the completion.
+ *
+ **/
 void
-ephy_location_entry_set_completion (EphyLocationEntry *le,
+ephy_location_entry_set_completion (EphyLocationEntry *entry,
 				    GtkTreeModel *model,
 				    guint text_col,
 				    guint action_col,
@@ -1010,75 +1183,85 @@ ephy_location_entry_set_completion (EphyLocationEntry *le,
 				    guint extra_col,
 				    guint favicon_col)
 {
-	EphyLocationEntryPrivate *priv = le->priv;
+	EphyLocationEntryPrivate *priv = entry->priv;
 	GtkTreeModel *sort_model;
 	GtkEntryCompletion *completion;
-	GtkCellRenderer *cell, *iconcell;
+	GtkCellRenderer *cell;
 
-	le->priv->text_col = text_col;
-	le->priv->action_col = action_col;
-	le->priv->keywords_col = keywords_col;
-	le->priv->relevance_col = relevance_col;
-	le->priv->url_col = url_col;
-	le->priv->extra_col = extra_col;
-	le->priv->favicon_col = favicon_col;
+	entry->priv->text_col = text_col;
+	entry->priv->action_col = action_col;
+	entry->priv->keywords_col = keywords_col;
+	entry->priv->relevance_col = relevance_col;
+	entry->priv->url_col = url_col;
+	entry->priv->extra_col = extra_col;
+	entry->priv->favicon_col = favicon_col;
 
 	sort_model = gtk_tree_model_sort_new_with_model (model);
-
 	g_object_unref (model);
+
 	gtk_tree_sortable_set_sort_column_id 
 			(GTK_TREE_SORTABLE (sort_model),
-			 le->priv->relevance_col,
+			 entry->priv->relevance_col,
 			 GTK_SORT_DESCENDING);
 
 	completion = gtk_entry_completion_new ();
 	gtk_entry_completion_set_model (completion, sort_model);
 	g_object_unref (sort_model);
 	g_signal_connect (completion, "match-selected",
-			  G_CALLBACK (match_selected_cb), le);
+			  G_CALLBACK (match_selected_cb), entry);
 	g_signal_connect_after (completion, "action-activated",
-				G_CALLBACK (action_activated_after_cb), le);
-
-	iconcell = gtk_cell_renderer_pixbuf_new ();
+				G_CALLBACK (action_activated_after_cb), entry);
 
+#if 0
+	cell = gtk_cell_renderer_pixbuf_new ();
 	gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (completion),
-				    iconcell, FALSE);
+				    cell, FALSE);
 	gtk_cell_layout_add_attribute (GTK_CELL_LAYOUT (completion),
-				       iconcell, "pixbuf", favicon_col);
+				       cell, "pixbuf", favicon_col);
+#endif
 
 	cell = gtk_cell_renderer_text_new ();
-	g_object_set (cell, 
-			"ellipsize", PANGO_ELLIPSIZE_END,
-			"ellipsize-set", TRUE,
-			NULL);
+	g_object_set (cell,
+		      "ellipsize", PANGO_ELLIPSIZE_END,
+		      "ellipsize-set", TRUE,
+		      NULL);
 	gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (completion),
 				    cell, TRUE);
 	gtk_cell_layout_add_attribute (GTK_CELL_LAYOUT (completion),
 				       cell, "text", text_col);
-
-	le->priv->extracell = gtk_cell_renderer_text_new ();
-	g_object_set (le->priv->extracell, 
-			"ellipsize", PANGO_ELLIPSIZE_END,
-			"ellipsize-set", TRUE,
-			"alignment", PANGO_ALIGN_LEFT,
-			NULL);
-
-	gtk_cell_layout_pack_end (GTK_CELL_LAYOUT (completion),
-				  le->priv->extracell, TRUE);
-	
+	gtk_cell_renderer_text_set_fixed_height_from_font (GTK_CELL_RENDERER_TEXT (cell), 2);
 	gtk_cell_layout_set_cell_data_func (GTK_CELL_LAYOUT (completion),
-					le->priv->extracell, extracell_data_func, 
-					le,
+					cell, textcell_data_func,
+					entry,
 					NULL);
 
+	cell = gtk_cell_renderer_pixbuf_new ();
+	gtk_cell_layout_pack_end (GTK_CELL_LAYOUT (completion),
+				  cell, FALSE);
+	gtk_cell_layout_set_cell_data_func (GTK_CELL_LAYOUT (completion),
+					    cell, extracell_data_func,
+					    entry,
+					    NULL);
+
 	g_object_set (completion, "inline-selection", TRUE, NULL);
 	g_signal_connect (completion, "cursor-on-match",
-			  G_CALLBACK (cursor_on_match_cb), le);
+			  G_CALLBACK (cursor_on_match_cb), entry);
 
-	gtk_entry_set_completion (GTK_ENTRY (priv->icon_entry->entry), completion);
+	gtk_entry_set_completion (GTK_ENTRY (priv->entry), completion);
 	g_object_unref (completion);
 }
 
+/**
+ * ephy_location_entry_set_location:
+ * @entry: an #EphyLocationEntry widget
+ * @address: the address to be set as the current location
+ * @typed_address: the address to be shown in the location entry or NULL
+ *
+ * Sets the current address of @entry to @address, alternatively a
+ * @typed_address can be used to determine the text to be displayed in the
+ * location entry.
+ *
+ **/
 void
 ephy_location_entry_set_location (EphyLocationEntry *entry,
 				  const char *address,
@@ -1097,19 +1280,17 @@ ephy_location_entry_set_location (EphyLocationEntry *entry,
 	 * bug #155824. So we save the selection iff the clipboard was owned by
 	 * the location entry.
 	 */
-	if (GTK_WIDGET_REALIZED (GTK_WIDGET (priv->icon_entry)))
+	if (GTK_WIDGET_REALIZED (GTK_WIDGET (priv->entry)))
 	{
-		GtkWidget *gtkentry = priv->icon_entry->entry;
-
-		clipboard = gtk_widget_get_clipboard (gtkentry,
+		clipboard = gtk_widget_get_clipboard (priv->entry,
 						      GDK_SELECTION_PRIMARY);
 		g_return_if_fail (clipboard != NULL);
 
-		if (gtk_clipboard_get_owner (clipboard) == G_OBJECT (gtkentry) &&
-		    gtk_editable_get_selection_bounds (GTK_EDITABLE (gtkentry),
+		if (gtk_clipboard_get_owner (clipboard) == G_OBJECT (priv->entry) &&
+		    gtk_editable_get_selection_bounds (GTK_EDITABLE (priv->entry),
 	     					       &start, &end))
 		{
-			selection = gtk_editable_get_chars (GTK_EDITABLE (gtkentry),
+			selection = gtk_editable_get_chars (GTK_EDITABLE (priv->entry),
 							    start, end);
 		}
 	}
@@ -1131,7 +1312,7 @@ ephy_location_entry_set_location (EphyLocationEntry *entry,
 	priv->hash = g_str_hash (address);
 
 	priv->block_update = TRUE;
-	gtk_entry_set_text (GTK_ENTRY (priv->icon_entry->entry), text);
+	gtk_entry_set_text (GTK_ENTRY (priv->entry), text);
 	priv->block_update = FALSE;
 
 	/* We need to call update_address_state() here, as the 'changed' signal
@@ -1150,6 +1331,16 @@ ephy_location_entry_set_location (EphyLocationEntry *entry,
 	}
 }
 
+/**
+ * ephy_location_entry_get_can_undo:
+ * @entry: an #EphyLocationEntry widget
+ *
+ * Wheter @entry can restore the displayed user modified text to the unmodified 
+ * previous text.
+ *
+ * Return value: TRUE or FALSE indicating if the text can be restored
+ *
+ **/
 gboolean
 ephy_location_entry_get_can_undo (EphyLocationEntry *entry)
 {
@@ -1158,6 +1349,16 @@ ephy_location_entry_get_can_undo (EphyLocationEntry *entry)
 	return priv->user_changed;
 }
 
+/**
+ * ephy_location_entry_get_can_redo:
+ * @entry: an #EphyLocationEntry widget
+ *
+ * Wheter @entry can restore the displayed text to the user modified version
+ * before the undo.
+ *
+ * Return value: TRUE or FALSE indicating if the text can be restored
+ *
+ **/
 gboolean
 ephy_location_entry_get_can_redo (EphyLocationEntry *entry)
 {
@@ -1166,12 +1367,22 @@ ephy_location_entry_get_can_redo (EphyLocationEntry *entry)
 	return priv->can_redo;
 }
 
+/**
+ * ephy_location_entry_get_location:
+ * @entry: an #EphyLocationEntry widget
+ *
+ * Retrieves the text displayed by the internal #GtkEntry of @entry. This is
+ * the currently displayed text, like in any #GtkEntry.
+ *
+ * Return value: the text inside the inner #GtkEntry of @entry, owned by GTK+
+ *
+ **/
 const char *
 ephy_location_entry_get_location (EphyLocationEntry *entry)
 {
 	EphyLocationEntryPrivate *priv = entry->priv;
 
-	return gtk_entry_get_text (GTK_ENTRY (priv->icon_entry->entry));
+	return gtk_entry_get_text (GTK_ENTRY (priv->entry));
 }
 
 static gboolean
@@ -1185,7 +1396,7 @@ ephy_location_entry_reset_internal (EphyLocationEntry *entry,
 
 	g_signal_emit (entry, signals[GET_LOCATION], 0, &url);
 	text = url != NULL ? url : "";
-	old_text = gtk_entry_get_text (GTK_ENTRY (priv->icon_entry->entry));
+	old_text = gtk_entry_get_text (GTK_ENTRY (priv->entry));
 	old_text = old_text != NULL ? old_text : "";
 
 	g_free (priv->saved_text);
@@ -1207,22 +1418,48 @@ ephy_location_entry_reset_internal (EphyLocationEntry *entry,
 	return retval;
 }
 
+/**
+ * ephy_location_entry_undo_reset:
+ * @entry: an #EphyLocationEntry widget
+ *
+ * Undo a previous ephy_location_entry_reset.
+ *
+ **/
 void
 ephy_location_entry_undo_reset (EphyLocationEntry *entry)
 {
 	EphyLocationEntryPrivate *priv = entry->priv;
 	
-	gtk_entry_set_text (GTK_ENTRY (priv->icon_entry->entry), priv->saved_text);
+	gtk_entry_set_text (GTK_ENTRY (priv->entry), priv->saved_text);
 	priv->can_redo = FALSE;
 	priv->user_changed = TRUE;
 }
 
+/**
+ * ephy_location_entry_reset:
+ * @entry: an #EphyLocationEntry widget
+ *
+ * Restore the @entry to the text corresponding to the current location, this
+ * does not fire the user_changed signal. This is called each time the user
+ * presses Escape while the location entry is selected.
+ *
+ * Return value: TRUE on success, FALSE otherwise
+ *
+ **/
 gboolean
 ephy_location_entry_reset (EphyLocationEntry *entry)
 {
 	return ephy_location_entry_reset_internal (entry, FALSE);
 }
 
+/**
+ * ephy_location_entry_activate:
+ * @entry: an #EphyLocationEntry widget
+ *
+ * Set focus on @entry and select the text whithin. This is called when the
+ * user hits Control+L.
+ *
+ **/
 void
 ephy_location_entry_activate (EphyLocationEntry *entry)
 {
@@ -1231,20 +1468,37 @@ ephy_location_entry_activate (EphyLocationEntry *entry)
 
 	toplevel = gtk_widget_get_toplevel (GTK_WIDGET (entry));
 
-	gtk_editable_select_region (GTK_EDITABLE(priv->icon_entry->entry),
+	gtk_editable_select_region (GTK_EDITABLE (priv->entry),
 				    0, -1);
-	gtk_window_set_focus (GTK_WINDOW(toplevel),
-			      priv->icon_entry->entry);
+	gtk_window_set_focus (GTK_WINDOW (toplevel),
+			      priv->entry);
 }
 
+/**
+ * ephy_location_entry_get_entry:
+ * @entry: an #EphyLocationEntry widget
+ *
+ * Retrieve the internal #GtkEntry of the #EphyIconEntry inside @entry.
+ *
+ * Return value: a pointer to the internal #GtkEntry of @entry
+ *
+ **/
 GtkWidget *
 ephy_location_entry_get_entry (EphyLocationEntry *entry)
 {
 	EphyLocationEntryPrivate *priv = entry->priv;
 
-	return priv->icon_entry->entry;
+	return priv->entry;
 }
 
+/**
+ * ephy_location_entry_set_favicon:
+ * @entry: an #EphyLocationEntry widget
+ * @pixbuf: a #GdkPixbuf to be set as the icon of the entry
+ *
+ * Sets the icon in the internal #EphyIconEntry of @entry
+ *
+ **/
 void
 ephy_location_entry_set_favicon (EphyLocationEntry *entry,
 				 GdkPixbuf *pixbuf)
@@ -1261,13 +1515,24 @@ ephy_location_entry_set_favicon (EphyLocationEntry *entry,
 	update_favicon (entry);
 }
 
+/**
+ * ephy_location_entry_set_secure:
+ * @entry: an #EphyLocationEntry widget
+ * @secure: whether the page is secure and thus the location bar should reflect
+ * that
+ *
+ * Set @entry to give visual feedback if the page is @secure. If it is, the
+ * location bar will have its background painted differently (yellow by
+ * default).
+ *
+ **/
 void
 ephy_location_entry_set_secure (EphyLocationEntry *entry,
 				gboolean secure)
 {
 	EphyLocationEntryPrivate *priv = entry->priv;
 	GtkWidget *widget = GTK_WIDGET (entry);
-	GtkWidget *gentry = ephy_icon_entry_get_entry (entry->priv->icon_entry);
+	GtkWidget *gentry = priv->entry;
 
 	priv->secure = secure;
 
@@ -1286,39 +1551,92 @@ ephy_location_entry_set_secure (EphyLocationEntry *entry,
 	gtk_widget_queue_draw (widget);
 }
 
+/**
+ * ephy_location_entry_set_show_lock:
+ * @entry: an #EphyLocationEntry widget
+ * @show_lock: if @entry should show a lock icon indicating the security level
+ * of the page
+ *
+ * If @show_lock is TRUE, the location bar will show an icon reflecting the
+ * security level of the page, by default it's shown only in secure and
+ * insecure pages (insecure meaning secure pages with something broken in such
+ * security)
+ *
+ **/
 void
 ephy_location_entry_set_show_lock (EphyLocationEntry *entry,
 				   gboolean show_lock)
 {
-	EphyLocationEntryPrivate *priv = entry->priv;
+	EphyLocationEntryPrivate *priv;
 
-	g_object_set (priv->lock_ebox, "visible", show_lock, NULL);
+	g_return_if_fail (EPHY_IS_LOCATION_ENTRY (entry));
+
+	priv = entry->priv;
+
+	priv->show_lock = show_lock != FALSE;
+
+	gtk_entry_set_icon_from_stock (GTK_ENTRY (priv->entry),
+				       GTK_ENTRY_ICON_SECONDARY,
+				       show_lock ? priv->lock_stock_id : NULL);
 }
 
+/**
+ * ephy_location_entry_set_lock_stock:
+ * @entry: an #EphyLocationEntry widget
+ * @stock_id: a stock_id from GTK+ stock icons
+ *
+ * Set the lock icon to be displayed, to actually show the icon see 
+ * ephy_location_entry_set_show_lock.
+ *
+ **/
 void
 ephy_location_entry_set_lock_stock (EphyLocationEntry *entry,
 				    const char *stock_id)
 
 {
-	EphyLocationEntryPrivate *priv = entry->priv;
+	g_return_if_fail (EPHY_IS_LOCATION_ENTRY (entry));
 
-	gtk_image_set_from_stock (GTK_IMAGE (priv->lock), stock_id,
-				  GTK_ICON_SIZE_MENU);
+	entry->priv->lock_stock_id = (char*)stock_id;
+
+	if (entry->priv->show_lock)
+		gtk_entry_set_icon_from_stock (GTK_ENTRY (entry->priv->entry),
+					       GTK_ENTRY_ICON_SECONDARY,
+					       stock_id);
 }
 
+/**
+ * ephy_location_entry_set_lock_tooltip:
+ * @entry: an #EphyLocationEntry widget
+ * @tooltip: the text to be set in the tooltip for the lock icon
+ *
+ * Set the text to be displayed when hovering the lock icon of @entry.
+ *
+ **/
 void
 ephy_location_entry_set_lock_tooltip (EphyLocationEntry *entry,
 				      const char *tooltip)
 {
 	EphyLocationEntryPrivate *priv = entry->priv;
 
-	gtk_widget_set_tooltip_text (priv->lock_ebox, tooltip);
+	gtk_entry_set_icon_tooltip_text (GTK_ENTRY (priv->entry),
+					 GTK_ENTRY_ICON_SECONDARY,
+					 tooltip);
 }
 
-GRegex *
-ephy_location_entry_get_regex (EphyLocationEntry *entry)
+/**
+ * ephy_location_entry_get_search_terms:
+ * @entry: an #EphyLocationEntry widget
+ *
+ * Return the internal #GSList containing the search terms as #GRegex
+ * instances, formed in @entry on user changes.
+ *
+ * Return value: the internal #GSList
+ *
+ **/
+GSList *
+ephy_location_entry_get_search_terms (EphyLocationEntry *entry)
 {
 	EphyLocationEntryPrivate *priv = entry->priv;
 	
-	return priv->regex;
+	return priv->search_terms;
 }
