@@ -16,7 +16,6 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- *  $Id$
  */
 
 #include "config.h"
@@ -127,8 +126,6 @@ create_tool_item (GtkAction *action)
 	gtk_widget_show (button);
 	gtk_container_add (GTK_CONTAINER (item), button);
 	g_object_set_data (G_OBJECT (item), "button", button);
-	gtk_activatable_set_related_action (GTK_ACTIVATABLE (button),
-					    action);
 
 	arrow = gtk_arrow_new (GTK_ARROW_DOWN, GTK_SHADOW_NONE);
 	gtk_widget_show (arrow);
@@ -377,7 +374,7 @@ button_enter_cb (GtkWidget *button,
 	    gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (active_button)))
 	{
 		EphyTopicAction *active_action;
-		active_action = gtk_widget_get_action (GTK_WIDGET (active_button));
+		active_action = (EphyTopicAction*)gtk_activatable_get_related_action (GTK_ACTIVATABLE (active_button));
 
 		erase_popup (active_action);
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (active_button), FALSE);
