@@ -17,7 +17,6 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- *  $Id$
  */
 
 #include "config.h"
@@ -27,6 +26,7 @@
 #include "ephy-embed.h"
 #include "ephy-embed-container.h"
 #include "ephy-embed-shell.h"
+#include "ephy-embed-utils.h"
 #include "ephy-file-helpers.h"
 #include "ephy-shell.h"
 #include "ephy-node.h"
@@ -37,6 +37,7 @@
 #include <gtk/gtk.h>
 #include <glib/gi18n.h>
 #include <string.h>
+#include <webkit/webkit.h>
 
 enum
 {
@@ -84,16 +85,23 @@ sync_encoding_against_embed (EphyEncodingDialog *dialog)
         GtkTreeModel *model;
         GList *rows;
 	GtkWidget *button;
-	char *encoding;
-	gboolean is_automatic;
+	const char *encoding;
+	gboolean is_automatic = FALSE;
+	WebKitWebView *view;
 
 	dialog->priv->update_tag = TRUE;
 
 	embed = ephy_embed_dialog_get_embed (EPHY_EMBED_DIALOG (dialog));
 	g_return_if_fail (EPHY_IS_EMBED (embed));
 
-	encoding = ephy_embed_get_encoding (embed);
-	if (encoding == NULL) return;
+	view = EPHY_GET_WEBKIT_WEB_VIEW_FROM_EMBED (embed);
+	encoding = webkit_web_view_get_custom_encoding (view);
+	if (encoding == NULL)
+	{
+		encoding = webkit_web_view_get_encoding (view);
+		if (encoding == NULL) return;
+		is_automatic = TRUE;
+	}
 
 	node = ephy_encodings_get_node (dialog->priv->encodings, encoding, TRUE);
 	g_assert (EPHY_IS_NODE (node));
@@ -119,13 +127,9 @@ sync_encoding_against_embed (EphyEncodingDialog *dialog)
 		g_list_free (rows);
 	}
 
-	is_automatic = ephy_embed_has_automatic_encoding (embed);
-
 	button = ephy_dialog_get_control (EPHY_DIALOG (dialog), properties[AUTOMATIC_PROP].id);
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), is_automatic);
 
-	g_free (encoding);
-
 	dialog->priv->update_tag = FALSE;
 }
 
@@ -197,6 +201,7 @@ activate_choice (EphyEncodingDialog *dialog)
 	EphyEmbed *embed;
 	GtkWidget *button;
 	gboolean is_automatic;
+	WebKitWebView *view;
 
 	embed = ephy_embed_dialog_get_embed (EPHY_EMBED_DIALOG (dialog));
 	g_return_if_fail (EPHY_IS_EMBED (embed));
@@ -204,9 +209,11 @@ activate_choice (EphyEncodingDialog *dialog)
 	button = ephy_dialog_get_control (EPHY_DIALOG (dialog), properties[AUTOMATIC_PROP].id);
 	is_automatic = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button));
 
+	view = EPHY_GET_WEBKIT_WEB_VIEW_FROM_EMBED (embed);
+
 	if (is_automatic)
 	{
-		ephy_embed_set_encoding (embed, "");
+		webkit_web_view_set_custom_encoding (view, NULL);
 	}
 	else if (dialog->priv->selected_node != NULL)
 	{
@@ -215,7 +222,7 @@ activate_choice (EphyEncodingDialog *dialog)
 		code = ephy_node_get_property_string (dialog->priv->selected_node,
 						      EPHY_NODE_ENCODING_PROP_ENCODING);
 
-		ephy_embed_set_encoding (embed, code);
+		webkit_web_view_set_custom_encoding (view, code);
 
 		ephy_encodings_add_recent (dialog->priv->encodings, code);
 	}
@@ -296,7 +303,7 @@ ephy_encoding_dialog_init (EphyEncodingDialog *dialog)
 
 	ephy_dialog_construct (EPHY_DIALOG (dialog),
 			       properties,
-			       ephy_file ("epiphany.glade"),
+			       ephy_file ("epiphany.ui"),
 			       "encoding_dialog",
 			       NULL);
 
