@@ -2,6 +2,7 @@
 /*
  *  Copyright © 2003, 2004 Marco Pesenti Gritti
  *  Copyright © 2003, 2004 Christian Persch
+ *  Copyright © 2008 Jan Alonzo
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,7 +18,6 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- *  $Id$
  */
 
 #include "config.h"
@@ -29,18 +29,26 @@
 #include "ephy-history.h"
 #include "ephy-embed-container.h"
 #include "ephy-embed-shell.h"
-#include "ephy-history-item.h"
+#include "ephy-embed-utils.h"
 #include "ephy-link.h"
 #include "ephy-gui.h"
 #include "ephy-debug.h"
 
 #include <gtk/gtk.h>
+#include <webkit/webkit.h>
 
-#define HISTORY_ITEM_DATA_KEY	"HistoryItem"
-#define URL_DATA_KEY	        "GoURL"
+#define HISTORY_ITEM_DATA_KEY	  "HistoryItem"
+#define URL_DATA_KEY	          "GoURL"
+#define WEBKIT_BACK_FORWARD_LIMIT 100
 
 #define EPHY_NAVIGATION_ACTION_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), EPHY_TYPE_NAVIGATION_ACTION, EphyNavigationActionPrivate))
 
+typedef enum
+{
+	WEBKIT_HISTORY_BACKWARD,
+	WEBKIT_HISTORY_FORWARD
+} WebKitHistoryType;
+
 struct _EphyNavigationActionPrivate
 {
 	EphyWindow *window;
@@ -108,7 +116,7 @@ static void
 activate_back_or_forward_menu_item_cb (GtkWidget *menuitem,
 				       EphyNavigationAction *action)
 {
-	EphyHistoryItem *item;
+	WebKitWebHistoryItem *item;
 	EphyEmbed *embed;
 
 	embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (action->priv->window));
@@ -121,26 +129,24 @@ activate_back_or_forward_menu_item_cb (GtkWidget *menuitem,
 		g_return_if_fail (embed != NULL);
 	}
 
-	item = (EphyHistoryItem*)g_object_get_data (G_OBJECT (menuitem), HISTORY_ITEM_DATA_KEY);
+	item = (WebKitWebHistoryItem*)g_object_get_data (G_OBJECT (menuitem), HISTORY_ITEM_DATA_KEY);
 	g_return_if_fail (item != NULL);
 
-	ephy_embed_go_to_history_item (embed, item);
+	webkit_web_view_go_to_back_forward_item (EPHY_GET_WEBKIT_WEB_VIEW_FROM_EMBED (embed), item);
 }
 
 static void
 select_menu_item_cb (GtkWidget *menuitem,
 	             EphyNavigationAction *action)
 {
-	char *url;
+	const char *url;
 	GtkWidget *statusbar;
-	EphyHistoryItem *item;
-	char *freeme = NULL;
+	WebKitWebHistoryItem *item;
 
-	item = (EphyHistoryItem*)g_object_get_data (G_OBJECT (menuitem), HISTORY_ITEM_DATA_KEY);
+	item = (WebKitWebHistoryItem*)g_object_get_data (G_OBJECT (menuitem), HISTORY_ITEM_DATA_KEY);
 	if (item)
 	{
-		url = ephy_history_item_get_url (item);
-		freeme = url;
+		url = webkit_web_history_item_get_uri (item);
 	}
 	else
 	{
@@ -151,8 +157,6 @@ select_menu_item_cb (GtkWidget *menuitem,
 	statusbar = ephy_window_get_statusbar (action->priv->window);
 
 	gtk_statusbar_push (GTK_STATUSBAR (statusbar), action->priv->statusbar_cid, url);
-
-	g_free (freeme);
 }
 
 static void
@@ -183,6 +187,24 @@ activate_up_menu_item_cb (GtkWidget *menuitem,
 			ephy_gui_is_middle_click () ? EPHY_LINK_NEW_TAB : 0);
 }
 
+static GList*
+webkit_construct_history_list (WebKitWebView *web_view, WebKitHistoryType hist_type) 
+{
+	WebKitWebBackForwardList *web_back_forward_list;
+	GList *webkit_items;
+
+	web_back_forward_list = webkit_web_view_get_back_forward_list (web_view);
+
+	if (hist_type == WEBKIT_HISTORY_FORWARD)
+		webkit_items = webkit_web_back_forward_list_get_forward_list_with_limit (web_back_forward_list,
+											 WEBKIT_BACK_FORWARD_LIMIT);
+	else
+		webkit_items = webkit_web_back_forward_list_get_back_list_with_limit (web_back_forward_list,
+										      WEBKIT_BACK_FORWARD_LIMIT);
+
+	return webkit_items;
+}
+
 static GtkWidget *
 build_back_or_forward_menu (EphyNavigationAction *action)
 {
@@ -190,14 +212,20 @@ build_back_or_forward_menu (EphyNavigationAction *action)
 	GtkMenuShell *menu;
 	EphyEmbed *embed;
 	GList *list, *l;
+	WebKitWebView *web_view;
 
 	embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (window));
 	g_return_val_if_fail (embed != NULL, NULL);
 
+	web_view = EPHY_GET_WEBKIT_WEB_VIEW_FROM_EMBED (embed);
+	g_return_val_if_fail (web_view != NULL, NULL);
+
 	if (action->priv->direction == EPHY_NAVIGATION_DIRECTION_BACK)
-		list = ephy_embed_get_backward_history (embed);
+		list = webkit_construct_history_list (web_view,
+						      WEBKIT_HISTORY_BACKWARD);
 	else
-		list = ephy_embed_get_forward_history (embed);
+		list = webkit_construct_history_list (web_view,
+						      WEBKIT_HISTORY_FORWARD);
 
 	menu = GTK_MENU_SHELL (gtk_menu_new ());
 
@@ -206,17 +234,15 @@ build_back_or_forward_menu (EphyNavigationAction *action)
 	for (l = list; l != NULL; l = l->next)
 	{
 		GtkWidget *item;
-		EphyHistoryItem *hitem;
-		char *title, *url;
+		WebKitWebHistoryItem *hitem;
+		const char *title, *url;
 
-		hitem = (EphyHistoryItem*)l->data;
-		url = ephy_history_item_get_url (hitem);
-		title = ephy_history_item_get_title (hitem);
+		hitem = (WebKitWebHistoryItem*)l->data;
+		url = webkit_web_history_item_get_uri (hitem);
+		title = webkit_web_history_item_get_title (hitem);
 
 		item = new_history_menu_item (title ? title : url, url);
 
-		g_free (title);
-
 		g_object_set_data_full (G_OBJECT (item), HISTORY_ITEM_DATA_KEY, hitem,
 					(GDestroyNotify) g_object_unref);
 
@@ -352,10 +378,13 @@ ephy_navigation_action_activate (GtkAction *gtk_action)
 	EphyNavigationAction *action = EPHY_NAVIGATION_ACTION (gtk_action);
 	EphyWindow *window = action->priv->window;
 	EphyEmbed *embed;
+	WebKitWebView *web_view;
 
 	embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (window));
 	g_return_if_fail (embed != NULL);
 
+	web_view = EPHY_GET_WEBKIT_WEB_VIEW_FROM_EMBED (embed);
+
 	if (action->priv->direction == EPHY_NAVIGATION_DIRECTION_BACK)
 	{
 		if (ephy_gui_is_middle_click ())
@@ -365,7 +394,7 @@ ephy_navigation_action_activate (GtkAction *gtk_action)
 						NULL,
 						EPHY_LINK_NEW_TAB);
 		}
-		ephy_embed_go_back (embed);
+		webkit_web_view_go_back (web_view);
 	}
 	else if (action->priv->direction == EPHY_NAVIGATION_DIRECTION_FORWARD)
 	{
@@ -376,7 +405,7 @@ ephy_navigation_action_activate (GtkAction *gtk_action)
 						NULL,
 						EPHY_LINK_NEW_TAB);
 		}
-		ephy_embed_go_forward (embed);
+		webkit_web_view_go_forward (web_view);
 	}
 	else if (action->priv->direction == EPHY_NAVIGATION_DIRECTION_UP)
 	{
